//
// To compile in Visual Studio 2022 (free Community Edition): 
//   1) Create a new Dynamic Link Library (DLL) C++ project
//   2) Copy this .c file into the Project Directory that was created in #1 above
//   3) Remove all Source Files added by #1 above (dllmain.cpp, psh.cpp) and Header Files (psh.h, framework.h)
//   4) Use Project -> Add Existing Item to select and add this .c file to the Source Files
//   5) Change Active Solution Platform from x64 to x86 in Configuration Manager (or toolbar)
//   6) With the project selected, turn off the Use Precompiled Headers option in Project -> Properties -> C/C++ -> Precompiled Heading section -> Precompiled Header -> Not Using Precompiled Headers
//   7) Choose Build Solution
//   8) .dll file compiled into \debug project directory will need to be copied to C:\NeuroShell Trader\Template directory
//      for the .dll file to be seen/used by a NeuroShell Trader chart.
//


#include <windows.h>   
#include <math.h>   

const float MISSING = (float) 3.4e38;    // NeuroShell Trader missing value constant

_declspec(dllexport) void EhlersLoops(float* Close, long LPPeriod, long HPPeriod, long numrows, float* output)
{
	// Ehlers Loops
	// (C) 2005 - 2022 John F.Ehlers

	double hpa1 = 0.0, hpb1 = 0.0, hpc1 = 0.0, hpc2 = 0.0, hpc3 = 0.0;
	double ssa1 = 0.0, ssb1 = 0.0, ssc1 = 0.0, ssc2 = 0.0, ssc3 = 0.0;
	double HP = 0.0, HP1 = 0.0, HP2 = 0.0;
	//double VolHP = 0.0, VolHP1 = 0.0, VolHP2 = 0.0;
	double Price = 0.0, Price1 = 0.0, Price2 = 0.0;
	double PriceMS = 0.0, PriceMS1 = 0.0;
	double PriceRMS = 0.0;
	//double Vol = 0.0, VolMS = 0.0, VolRMS = 0.0;
	double radiansconvertmult = 3.1415926535 / 180.0;
	long Currentbar;

	hpa1 = exp(-1.414 * 3.14159 / HPPeriod);
	hpb1 = 2 * hpa1 * cos(radiansconvertmult * 1.414 * 180 / HPPeriod);
	hpc2 = hpb1;
	hpc3 = -hpa1 * hpa1;
	hpc1 = (1 + hpc2 - hpc3) / 4;
	ssa1 = exp(-1.414 * 3.14159 / LPPeriod);
	ssb1 = 2 * ssa1 * cos(radiansconvertmult * 1.414 * 180 / LPPeriod);
	ssc2 = ssb1;
	ssc3 = -ssa1 * ssa1;
	ssc1 = 1 - ssc2 - ssc3;

	for (Currentbar = 0; Currentbar < numrows; Currentbar++)
	{
		if (Currentbar >= 3)
		{	//Normalized Roofing Filter for Price
			// 2 Pole Butterworth Highpass Filter
			HP = hpc1 * (Close[Currentbar] - 2 * Close[Currentbar-1] + Close[Currentbar-2]) + hpc2 * HP1 + hpc3 * HP2;

			// Smooth with a Super Smoother Filter
			Price = ssc1 * (HP + HP1) / 2 + ssc2 * Price1 + ssc3 * Price2;

			//Scale Price in terms of Standard Deviations
			PriceMS = .0242 * Price * Price + .9758 * PriceMS1;

			if (PriceMS != 0)
				PriceRMS = Price / sqrt(PriceMS);

			output[Currentbar] = (float)PriceRMS;

			HP2 = HP1;
			HP1 = HP;
			Price2 = Price1;
			Price1 = Price;
			PriceMS1 = PriceMS;

			// ----------------------------------------

			//Normalized Roofing Filter for Volume
			// 2 Pole Butterworth Highpass Filter
			//VolHP = hpc1 * (Volume - 2 * Volume[1] + Volume[2]) + hpc2 * VolHP[1] + hpc3 * VolHP[2];

			// Smooth with a Super Smoother Filter
			//Vol = ssc1 * (VolHP + VolHP[1]) / 2 + ssc2 * Vol[1] + ssc3 * Vol[2];

			//Scale Vol in terms of Standard Deviations
			//VolMS = .0242 * Vol * Vol + .9758 * VolMS[1];
			//if (VolMS != 0)
			//	VolRMS = Vol / SquareRoot(VolMS);
		}
	}
}
